__version__ = "6.0.5"
__author__ = "Nico Schlömer"
__author_email__ = "nico.schloemer@gmail.com"
__copyright__ = f"Copyright (c) 2013-2020, {__author__} <{__author_email__}>"
__website__ = "https://github.com/nschloe/pygmsh"
__license__ = "License :: OSI Approved :: MIT License"
__status__ = "Development Status :: 5 - Production/Stable"
